import java.util.ArrayList;
import java.lang.reflect.*;

public class Kontrolor {
    private Class trieda, rodic;
    private Package balicek;
    private Class[] rozhrania;
    private ArrayList<String> kniznice;
     
    public Kontrolor(Object instancia) {
        if (instancia != null)
            this.trieda = instancia.getClass();
        kniznice = new ArrayList<>();
    }
    
    public void kontroluj() {
        if (trieda != null) {
            balicek = trieda.getPackage();
            rodic = trieda.getSuperclass();
            rozhrania = trieda.getInterfaces();

            String telo = vypisHlavicku() + vypisAtributy() + 
                          vypisKonstruktory() + vypisMetody();


            String kod = "";
            if (balicek != null) 
                kod += "package " + balicek.getName() + ";\n\n";
            for (int i = 0; i < kniznice.size(); i++) {
                kod += "import " + kniznice.get(i) + ";\n";
                if (i + 1 == kniznice.size())
                    kod += "\n";
            }
            kod += telo;
            kod += "}";
            System.out.println(kod);
        }
    }
    
    private String vypisHlavicku() {
        String s = "";
        if (Modifier.isPublic(trieda.getModifiers()))
            s += "public ";
        else if (Modifier.isProtected(trieda.getModifiers()))
            s += "protected ";
        else
            s += "private ";
     
        s += "class " + dajNazov(trieda.getName());
        if (rodic != null) {
            String r = " extends " + dajNazov(rodic.getName());
            if (!(r.equals(" extends Object")))
                s += r;
        }
        if (rozhrania.length > 0) {
            s += " implenets ";
            for (int i = 0; i < rozhrania.length; i++) {
                s += dajNazov(rozhrania[i].getName());
                if (i + 1 != rozhrania.length)
                    s+= ", ";
            } 
        }
        return s + " {\n";
    }  
    private String vypisAtributy() {
        String s = "";
        Field[] f = trieda.getDeclaredFields();
        for (int i = 0; i <  f.length; i++) {
            String[] pole = f[i].toGenericString().split(" ");
            pole[pole.length - 2] = dajNazov(pole[pole.length - 2]);
            pole[pole.length - 1] = pole[pole.length - 1].replace(trieda.getName() + ".", "");
            s += "\t";
            for (int j = 0; j < pole.length; j++) {
                s += pole[j];
                if (j + 1 != pole.length)
                    s += " ";
            }
            s += ";\n";
        }
        return s + "\n";
    }
    private String vypisKonstruktory() {
        Constructor[] c = trieda.getDeclaredConstructors();
        String s = "";
        for (int i = 0; i < c.length; i++) {
            s += vypisParametre("\t" + c[i].toGenericString().replace(trieda.getName() + ".", "").replace("java.lang.", ""), c[i].getParameters());
            s += vypisVynimky(c[i].getExceptionTypes());
            s += ";\n";
        }
        s += "\n";
        return s;
    }
    private String vypisMetody() {
        String s = "";
        Method[] m = trieda.getDeclaredMethods();
        for (int i = 0; i < m.length; i++) {
            s += vypisParametre("\t" + m[i].toGenericString().replace(trieda.getName() + ".", "").replace("java.lang.", ""), m[i].getParameters());
            s += vypisVynimky(m[i].getExceptionTypes());
            s += ";\n";
        }
        return s;
    }
    private String vypisParametre(String metoda, Parameter[] parametre) {
        String s = metoda.substring(0, metoda.indexOf("(") + 1);
        metoda = metoda.substring(metoda.indexOf("(") + 1, metoda.indexOf(")"));
        String[] pole = metoda.split(",");
        String p;
        for (int i = 0; i < parametre.length; i++) {
            s += dajNazov(pole[i]) + " " + parametre[i].getName(); //nove JDK8 ma vypisovat nazvy parametrov, starsie verzie Javy by mali vypisat arg0, arg1...
            if (i + 1 != parametre.length)
                s += ", ";
        }
        return s + ")";
    }
    private String vypisVynimky(Class[] vynimky) {
        String s = "";
        if (vynimky.length > 0) {
            s+= " throws ";
            for (int i = 0; i < vynimky.length; i++) {
                s += dajNazov(vynimky[i].getTypeName());
                if (i + 1 != vynimky.length)
                    s += ", ";
            }      
        }
        return s;
    }
    private String dajNazov(String nazov) {
        if (nazov.indexOf(".") > 0) {
            if (balicek != null && nazov.indexOf(balicek.getName()) == 0) {
                return nazov.replace(balicek.getName() + ".", "");
            } 
            if (!obsahuje(nazov.replace("[]", "")) && nazov.indexOf("<") == -1)
                kniznice.add(nazov.replace("[]", ""));
            else if (nazov.indexOf("<") > 0) {
                nazov = nazov.replace(">", "");
                String[] pole = nazov.split("<");
                return dajNazov(pole[0]) + "<" + dajNazov(pole[1]) + ">";
            }
            return nazov.replace(nazov.substring(0, nazov.lastIndexOf(".")) + ".", "");     
        }
        return nazov;
    }
    private boolean obsahuje(String kniznica) {
        if (kniznica.indexOf("java.lang.") == 0) {
            int bodka = 2;
            for (int i = 10; i < kniznica.length(); i++)
                if (kniznica.charAt(i) == '.')
                    bodka++;
            if (bodka == 2)
                return true;
        }  
        for (String k : kniznice)
            if (k.equals(kniznica))
                return true;
        return false;
    }
    
    public static void main(String[] args) {
        // v pravidla bolo, ze program musi mat spustitelnu verziu, tak som spravil jednoduchu ukazku
        Produkt p = new Produkt("banán", "5ks", "chutné", "cesta", 5, "1, 5, 7", "šupka");
        Kontrolor k = new Kontrolor(p);
        k.kontroluj();
    }
}
